package ir.ac.iust.htmlchardet.langwiseeval;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileAlreadyExistsException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.commons.jcs.utils.zip.CompressionUtil;
import org.apache.log4j.Logger;

// @formatter:off
/**
 * 
 * @author shabanali faghani
 * 
 * This class uses one connection for each DB to perform SQL instructions.
 * 
 * As stated in these pages:
 * 1. http://www.sqlite.org/threadsafe.html
 * 2. http://stackoverflow.com/questions/10707434/sqlite-in-a-multithreaded-java-application
 * using single connection in a multi-threaded environment with SQLite default configuration is thread-safe.
 * 
 * If performance is concerned, there are some helpful options here:
 * http://stackoverflow.com/questions/5181927/efficient-way-to-ensure-unique-rows-in-sqlite3
 *
 * To use the code of this class for reading from table[s] you should make some changes and also add some code.
 */
// @formatter:on
public class CorpusDBManager {

	private static final Logger LOG = Logger.getLogger(CorpusDBManager.class);

	private String language;
	private static CorpusDBManager INSTANCE = null;
	private static Connection langDBConnection = null;
	private static Connection resultDBConnection = null;
	private String corpusPath = "corpus/";
	private boolean isCompressHTML = true;

	public static CorpusDBManager getInstance(String language) {
		if (INSTANCE == null) {
			INSTANCE = new CorpusDBManager(language);
		}
		return INSTANCE;
	}

	private CorpusDBManager(String language) {
		super();
		this.language = language;
		String dbName = corpusPath + this.language + ".db";
		LOG.info("Creating " + dbName + " ...");
		try {
			File langDB = new File(dbName);
			if (langDB.exists()) {
				throw new FileAlreadyExistsException("The file at path " + dbName + " is already exists.");
			}
			Class.forName("org.sqlite.JDBC");
			langDBConnection = createLangDB(dbName);
			resultDBConnection = prepareResultsDB();
		} catch (Exception e) {
			LOG.fatal(e.getClass().getName() + ": " + e.getMessage(), e);
			System.exit(1);
		}
		LOG.info("Creating " + dbName + " done successfully");
	}

	private Connection createLangDB(String dbPath) throws ClassNotFoundException, SQLException {
		Connection connection = DriverManager.getConnection("jdbc:sqlite:" + dbPath);
		Statement statement = connection.createStatement();
		String sql = "CREATE TABLE IF NOT EXISTS " + this.language +
						" (docId	INTEGER PRIMARY KEY NOT NULL," + 
						" url 		TEXT NOT NULL," + 
						" charset 	TEXT NOT NULL," +
						" html 		BLOB NOT NULL);";
		statement.executeUpdate(sql);
		statement.close();
		return connection;
	}

	private Connection prepareResultsDB() throws ClassNotFoundException, SQLException {
		Connection connection = DriverManager.getConnection("jdbc:sqlite:" + corpusPath + "AGGREGATED-RESULTS.db");
		Statement statement = connection.createStatement();
		// delete results of the previous run for current language, if it has been ever ran
		String sql = "DELETE FROM results WHERE language = ?";
		PreparedStatement prepareStatement = connection.prepareStatement(sql);
		prepareStatement.setString(1, this.language);
		prepareStatement.executeUpdate();
		statement.close();
		return connection;
	}

	public void insertPage(int docId, String url, String charset, byte[] html) throws IOException, SQLException {
		if (isCompressHTML) {
			html = CompressionUtil.compressByteArray(html);
		}
		String sql = "INSERT INTO " + this.language + " (docId, url, charset, html) VALUES (?, ?, ?, ?)";
		PreparedStatement statement = langDBConnection.prepareStatement(sql);
		statement.setInt(1, docId);
		statement.setString(2, url);
		statement.setString(3, charset);
		statement.setBytes(4, html);

		statement.executeUpdate();
		statement.close();		// OOM error :(( several hours wasted because of commenting these lines
	}

	public void insertResult(int docId, String url, String httpCharset, String iust, String icu4jStripped, String tika,
			String iustWithJUniChar, String jUniCharDet, String iustWithoutMarkupElim) throws SQLException {
		String sql = "INSERT INTO results (language, docId, url, httpCharset, iust, icu4jStripped, tika, iustWithJUniChar, jUniCharDet, iustWithoutMarkupElim) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		PreparedStatement statement = resultDBConnection.prepareStatement(sql);
		statement.setString(1, this.language);
		statement.setInt(2, docId);
		statement.setString(3, url);
		statement.setString(4, httpCharset);
		statement.setString(5, iust);
		statement.setString(6, icu4jStripped);
		statement.setString(7, tika);
		statement.setString(8, iustWithJUniChar);
		statement.setString(9, jUniCharDet);
		statement.setString(10, iustWithoutMarkupElim);

		statement.executeUpdate();
		statement.close();		// OOM!
	}

	public static void closeConnections() {
		try {
			langDBConnection.close();
			resultDBConnection.close();
		} catch (SQLException e) {
			LOG.error("Could not close JDBC connection[s]", e);
		}
	}

}

