package ir.ac.iust.htmlchardet.langwiseeval;

import ir.ac.iust.htmlchardet.Charsets;
import ir.ac.iust.htmlchardet.HTMLCharsetDetector;

import java.io.ByteArrayInputStream;
import java.net.URL;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.apache.tika.detect.AutoDetectReader;
import org.jsoup.Jsoup;
import org.jsoup.helper.HttpConnection.Response;

/**
 * 
 * @author shabanali faghani
 * 
 */
public class LangCrawlThread extends Thread {

	private static final Logger LOG = Logger.getLogger(LangCrawlThread.class);

	private BlockingQueue<String> langURLQueue = null;
	private ConcurrentMap<String, ConcurrentMap<String, Integer>> detectedCharsetStat = null;

	private AtomicInteger urlCounter = null;
	private AtomicInteger validHTTPCharset = null;
	private AtomicInteger fetchedPages = null;

	private CorpusDBManager corpusDBManager = null;

	private int connectionTimeout;
	private boolean isStop = false;

	public LangCrawlThread(String language, String threadName, BlockingQueue<String> langURLQueue,
			ConcurrentMap<String, ConcurrentMap<String, Integer>> detectedCharsetStat) {
		super(threadName);
		this.corpusDBManager = CorpusDBManager.getInstance(language);
		this.langURLQueue = langURLQueue;
		this.detectedCharsetStat = detectedCharsetStat;
	}

	@Override
	public void run() {
		while (!langURLQueue.isEmpty() && !isStop) {
			try {
				String url = langURLQueue.remove();
				urlCounter.incrementAndGet();
				url = "http://www." + url;
				// maybe we should make follow redirects false!
				Response response = (Response) Jsoup.connect(url).followRedirects(true).timeout(connectionTimeout * 1000).execute();
				String httpCharset = response.charset();
				if (Charsets.isValid(httpCharset)) {
					validHTTPCharset.incrementAndGet();
					httpCharset = Charsets.normalize(httpCharset);
					byte[] htmlByteSequence = IOUtils.toByteArray(new URL(url));
					int docId = fetchedPages.incrementAndGet();

					String iust = HTMLCharsetDetector.detect(htmlByteSequence, false);
					String icu4jStripped = ICU4jWithStrippingEncodingDetector.detect(htmlByteSequence);
					String tika = new AutoDetectReader(new ByteArrayInputStream(htmlByteSequence)).getCharset().toString();
					String iustWithJUniChar = IUSTWithJUniversalEncodingDetector.detect(htmlByteSequence);
					String jUniCharDet = JUniversalEncodingDetector.detect(htmlByteSequence);
					String iustWithoutMarkupElim = IUSTWithoutMarkupElimination.detect(htmlByteSequence);

					// update results for online/interactive user
					updateDetectedCharsetStat(httpCharset, iust, Main.iustHTMLCharDetStat);
					updateDetectedCharsetStat(httpCharset, icu4jStripped, Main.icu4jStripped);
					updateDetectedCharsetStat(httpCharset, tika, Main.tikaLegacyAlgorithm);
					updateDetectedCharsetStat(httpCharset, iustWithJUniChar, Main.iustWithJUniChar);
					updateDetectedCharsetStat(httpCharset, jUniCharDet, Main.jUniversalCharDet);
					updateDetectedCharsetStat(httpCharset, iustWithoutMarkupElim, Main.iustWithoutMarkupElim);

					// save the page in corpus
					corpusDBManager.insertPage(docId, url, httpCharset, htmlByteSequence);
					// save results in results.db
					corpusDBManager.insertResult(docId, url, httpCharset, iust, icu4jStripped, tika, iustWithJUniChar, jUniCharDet, iustWithoutMarkupElim);

				} else if (httpCharset != null && !httpCharset.isEmpty()) {
					// log into log/unsupported-charsets.log
					LOG.info("Charset in HTTP header: " + httpCharset + "\tURL: " + url);
				}
			} catch (Throwable t) {
				LOG.error(this.getName() + " " + t);
			}
		}
	}

	private void updateDetectedCharsetStat(String charset, String detectedCharset, String detectorStat) {
		String pair = charset + "->" + detectedCharset;
		if (detectedCharsetStat.get(detectorStat).containsKey(pair)) {
			int preStat = detectedCharsetStat.get(detectorStat).get(pair);
			detectedCharsetStat.get(detectorStat).put(pair, preStat + 1);
		} else {
			detectedCharsetStat.get(detectorStat).put(pair, 1);
		}
	}

	public boolean stopCrawl() {
		isStop = true;
		return true;
	}

	public LangCrawlThread setConnectionTimeout(int connectionTimeout) {
		this.connectionTimeout = connectionTimeout;
		return this;
	}

	public LangCrawlThread setUrlCounter(AtomicInteger urlCounter) {
		this.urlCounter = urlCounter;
		return this;
	}

	public LangCrawlThread setValidHTTPCharsetCounter(AtomicInteger validHTTPCharset) {
		this.validHTTPCharset = validHTTPCharset;
		return this;
	}

	public LangCrawlThread setFetchedPagesCounter(AtomicInteger fetchedPages) {
		this.fetchedPages = fetchedPages;
		return this;
	}

}
