package ir.ac.iust.htmlchardet.langwiseeval;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * 
 * @author shabanali faghani
 * 
 */
public class Main {

	private static final Logger LOG = Logger.getLogger(Main.class);

	private static ConcurrentMap<String, ConcurrentMap<String, Integer>> detectedCharsetStat = null;
	private static AtomicInteger urlCounter = new AtomicInteger(0);
	private static AtomicInteger validHTTPCharset = new AtomicInteger(0);
	private static AtomicInteger fetchedPages = new AtomicInteger(0);

	public static final String iustHTMLCharDetStat = "iust";
	public static final String icu4jStripped = "icu4jStripped";
	public static final String tikaLegacyAlgorithm = "tika";
	public static final String iustWithJUniChar = "iustWithJUniChar";
	public static final String jUniversalCharDet = "jUniCharDet";
	public static final String iustWithoutMarkupElim = "iustWithoutMarkupElim";

	private static void setUp() {
		PropertyConfigurator.configure("log/log4j.properties");
		detectedCharsetStat = new ConcurrentHashMap<String, ConcurrentMap<String, Integer>>();
		detectedCharsetStat.put(iustHTMLCharDetStat, new ConcurrentHashMap<String, Integer>());
		detectedCharsetStat.put(icu4jStripped, new ConcurrentHashMap<String, Integer>());
		detectedCharsetStat.put(tikaLegacyAlgorithm, new ConcurrentHashMap<String, Integer>());
		detectedCharsetStat.put(iustWithJUniChar, new ConcurrentHashMap<String, Integer>());
		detectedCharsetStat.put(jUniversalCharDet, new ConcurrentHashMap<String, Integer>());
		detectedCharsetStat.put(iustWithoutMarkupElim, new ConcurrentHashMap<String, Integer>());
	}

	public static void main(String[] args) throws InterruptedException, IOException {
		if (args.length != 4) {
			System.out.println("Arguments aren't correct!");
			System.out.println("You have to run this program with 4 argument as follows:");
			System.out.println("java -jar lang-wise-eval.jar language numofThreads connectionTimeout(seconds) showStatInterval(seconds)");
			System.out.println("For example: java -jar lang-wise-eval.jar Japanese 100 60 20");
			System.out.println("P.S. language argument is case-sensitive (the first letter has to be in UPPER-CASE)");
			System.exit(1);
		}
		
		String language = args[0];
		int numOfThreads = Integer.parseInt(args[1]);
		int connectionTimeout = Integer.parseInt(args[2]);
		int showStatInterval = Integer.parseInt(args[3]);
		
		setUp();

		BlockingQueue<String> langURLQueue = new ArrayBlockingQueue<String>(50000);
		langURLQueue.addAll(FileUtils.readLines(new File("corpus/urls/" + language)));

		List<LangCrawlThread> crawlThreads = new ArrayList<LangCrawlThread>();
		for (int i = 0; i < numOfThreads; i++) {
			LangCrawlThread thread = new LangCrawlThread(language, "Crawl-Thread-" + i, langURLQueue, detectedCharsetStat);
			thread.setConnectionTimeout(connectionTimeout)
					.setUrlCounter(urlCounter)
					.setValidHTTPCharsetCounter(validHTTPCharset)
					.setFetchedPagesCounter(fetchedPages);
			crawlThreads.add(thread);
			thread.start();
			LOG.info("Thread " + thread.getName() + " got launched!");
		}

		// shutdown hook doesn't works with eclipse RED Terminate button but works by
		// Ctrl + C in command line.
		Runtime.getRuntime().addShutdownHook(new CrawlStopperHook(crawlThreads));

		do {
			Thread.sleep(showStatInterval * 1000);
			showCurrentStats();
		} while (langURLQueue.size() > 0);

		for (LangCrawlThread langCrawlThread : crawlThreads) {
			langCrawlThread.join();
		}
		showCurrentStats();
	}

	/**
	 * shows the current statistics/state in console for online user
	 */
	private static void showCurrentStats() {
		Map<String, Integer> accuracy = new HashMap<String, Integer>();
		detectedCharsetStat.forEach((k, v) -> {
			System.out.println(k + ":");
			accuracy.put(k, 0);
			v.forEach((k2, v2) -> {
				System.out.println(k2 + ":\t" + v2);
				String pair[] = k2.split("->");
				if (pair[0].equalsIgnoreCase(pair[1])
						|| (pair[0].equalsIgnoreCase("GB2312") && pair[1].equalsIgnoreCase("GB18030"))
						|| (pair[0].equalsIgnoreCase("GBK") && pair[1].equalsIgnoreCase("GB18030")))
					accuracy.put(k, accuracy.get(k) + v2);
			});
			System.out.println("--------------------------------------------");
		});
		System.out.println("****************************************************************");
		System.out.println("#processed URLs:\t" + urlCounter.get());
		System.out.println("#pages with valid charset in HTTP header:\t" + validHTTPCharset.get());
		System.out.println("#fetched pages:\t" + fetchedPages.get());
		System.out.println("--------------------------------------------");
		System.out.println("Accuracy:");
		accuracy.forEach((k, v) -> System.out.print(k + ": " + v + "  "));
		System.out.println(); // go to new line
		System.out.println("Accuracy(%):");
		accuracy.forEach((k, v) -> System.out.print(k + ": " + v * 100 / (fetchedPages.get() != 0 ? fetchedPages.get() : 1) + "%  "));
		System.out.println(); // go to new line
		System.out.println("**************************************************************************************************");
	}

}
